==> 0.0.3.tar.gz: lib/create_feature/create_feature_cli.dart <==
import 'dart:async';
import 'dart:io';

import 'package:path/path.dart' as path;

import '../util/utilities.dart';

Future<void> createNewFeature(String featureName) async {
  String destinationPath = 'lib/features/$featureName';
  String currentFilePath = Platform.script.toFilePath();
  List<String> pathSegments = currentFilePath.split(Platform.pathSeparator);
  if (pathSegments.length > 2) {
    pathSegments.removeRange(pathSegments.length - 5, pathSegments.length);
  }

  String packageName = 'jt_flutter_cli';
  String version = getPackageVersion(packageName);
  String dependencyName = '$packageName-$version';
  String pubCachePath = Platform.environment['PUB_CACHE'] ??
      '${Platform.environment['HOME']}/.pub-cache';
  String dependencyPath =
      path.join(pubCachePath, 'hosted', 'pub.dev', dependencyName);
  if (!Directory(dependencyPath).existsSync()) {
    print('Dependency $dependencyName not found in .pub-cache directory');
    return;
  }
  String dependencyLibPath = path.join(dependencyPath, 'lib');
  if (!Directory(dependencyLibPath).existsSync()) {
    print('Dependency $dependencyName does not contain a lib folder');
    return;
  }
  String sourceFolderPath = '$dependencyLibPath/create_feature/basic_structure';
  grantPermissions(sourceFolderPath);

  if (!Directory(sourceFolderPath).existsSync()) {
    print('Error: $sourceFolderPath folder does not exist');
    return;
  }

  Directory destinationDirectory = Directory(destinationPath);
  if (destinationDirectory.existsSync()) {
    print(
        'Error: Destination feature directory already exists. Please choose a different feature name');
    return;
  }

  try {
    await copyFolder(sourceFolderPath, destinationPath);
    processFiles(destinationPath, '/*', '', false);
    processFiles(destinationPath, '*/', '', false);
    renameFilesWithSubstring(destinationPath, 'basic_structure', featureName);
    processFiles(destinationPath, 'BasicStructure', featureName, true);
    processFiles(
        destinationPath, 'basic_structure', featureName.toLowerCase(), false);
    String? name = getPubspecInfo('name');
    if (name != null) {
      processFiles(destinationPath, 'flutter_clean_architecture', name, false);
    }

    final nameOfFeature = featureName.toLowerCase();
    String pascalCase = toPascalCase(nameOfFeature);
    final filePath = 'lib/core/di/';
    final repoImport =
        "import 'package:${name ?? ''}/features/$nameOfFeature/domain/repositories/${nameOfFeature}_repo.dart';";

    final diGetterImports = [repoImport];
    final diGetterStatements = ["${pascalCase}Repo get${pascalCase}Repo();"];
    addImportAndChangesInFile(
        '$filePath/di_getter.dart', diGetterImports, diGetterStatements);

    final mockDiGetterImports = [
      "import 'package:${name ?? ''}/features/$nameOfFeature/data/repository/mock_${nameOfFeature}_repo_impl.dart';",
      repoImport
    ];
    final mockDiGetterStatements = [
      "${pascalCase}Repo get${pascalCase}Repo() => Mock${pascalCase}RepoImpl(sl(), sl());"
    ];
    addImportAndChangesInFile('$filePath/mock_di_getter.dart',
        mockDiGetterImports, mockDiGetterStatements);

    final prodDiGetterImports = [
      "import 'package:${name ?? ''}/features/$nameOfFeature/data/repository/${nameOfFeature}_repo_impl.dart';",
      repoImport
    ];
    final prodDiGetterStatements = [
      "${pascalCase}Repo get${pascalCase}Repo() => ${pascalCase}RepoImpl(sl(), sl());"
    ];
    addImportAndChangesInFile('$filePath/prod_di_getter.dart',
        prodDiGetterImports, prodDiGetterStatements);

    final injectionContainerImports = [
      "import 'package:${name ?? ''}/features/$nameOfFeature/data/data_source/${nameOfFeature}_data_source.dart';",
      "import 'package:${name ?? ''}/features/$nameOfFeature/data/data_source/${nameOfFeature}_data_source_impl.dart';"
    ];
    final injectionContainerStatements = [
      "sl.registerLazySingleton<${pascalCase}DataSource>("
          "() => ${pascalCase}DataSourceImpl(remoteDataSource: sl()),"
          ");"
    ];
    addImportAndChangesInFile('$filePath/injection_container.dart',
        injectionContainerImports, injectionContainerStatements);
    print('Feature $featureName created');
  } catch (e) {
    print('Error copying folder: $e');
  }
}

Future<void> addImportAndChangesInFile(String filePath,
    List<String> importStatements, List<String> methodCalls) async {
  final file = File(filePath);
  final lines = await file.readAsLines();

  int lastImportIndex = -1;
  for (int i = 0; i < lines.length; i++) {
    if (lines[i].startsWith("import ")) {
      lastImportIndex = i;
    } else {
      break;
    }
  }

  if (lastImportIndex != -1) {
    if (importStatements.isNotEmpty) {
      lines.insertAll(lastImportIndex + 1, importStatements);
    }
    int completionIndex = -1;
    for (int i = 0; i < lines.length; i++) {
      if (lines[i].trim() == '}') {
        completionIndex = i;
        break;
      }
    }

    if (completionIndex != -1) {
      if (methodCalls.isNotEmpty) {
        lines.insertAll(completionIndex, methodCalls);
      }
      await file.writeAsString(lines.join('\n'));
    }
  }
}

==> 0.0.9.tar.gz: lib/create_feature/create_feature_cli.dart <==
import 'dart:async';
import 'dart:io';

import 'package:path/path.dart' as path;

import '../util/utilities.dart';

Future<void> createNewFeature(String featureName) async {
  String destinationPath = 'lib/features/$featureName';
  String currentFilePath = Platform.script.toFilePath();
  List<String> pathSegments = currentFilePath.split(Platform.pathSeparator);
  if (pathSegments.length > 2) {
    pathSegments.removeRange(pathSegments.length - 5, pathSegments.length);
  }

  String packageName = 'jt_flutter_cli';
  String version = getPackageVersion(packageName);
  String dependencyName = '$packageName-$version';

  String pubCachePath;
  if (Platform.isWindows) {
    String userprofile = Platform.environment['USERPROFILE'] ?? '';
    print("path : $userprofile");
    pubCachePath = Platform.environment['PUB_CACHE'] ??
        path.join(userprofile, 'AppData', 'Local', 'Pub', 'Cache');
    print("cache path  : $pubCachePath");
  } else {
    pubCachePath = Platform.environment['PUB_CACHE'] ??
        path.join(Platform.environment['HOME']!, '.pub-cache');
  }
  String dependencyPath =
      path.join(pubCachePath, 'hosted', 'pub.dev', dependencyName);
  if (!Directory(dependencyPath).existsSync()) {
    print('Dependency $dependencyName not found in .pub-cache directory');
    return;
  }
  String dependencyLibPath = path.join(dependencyPath, 'lib');
  if (!Directory(dependencyLibPath).existsSync()) {
    print('Dependency $dependencyName does not contain a lib folder');
    return;
  }
  String sourceFolderPath = '$dependencyLibPath/create_feature/basic_structure';
  grantPermissions(sourceFolderPath);

  if (!Directory(sourceFolderPath).existsSync()) {
    print('Error: $sourceFolderPath folder does not exist');
    return;
  }

  Directory destinationDirectory = Directory(destinationPath);
  if (destinationDirectory.existsSync()) {
    print(
        'Error: Destination feature directory already exists. Please choose a different feature name');
    return;
  }

  try {
    await copyFolder(sourceFolderPath, destinationPath);
    await processFiles(destinationPath, '/*', '', false);
    await processFiles(destinationPath, '*/', '', false);
    await renameFilesWithSubstring(
        destinationPath, 'basic_structure', featureName);
    await processFiles(destinationPath, 'BasicStructure', featureName, true);
    await processFiles(
        destinationPath, 'basic_structure', featureName.toLowerCase(), false);
    String? name = getPubspecInfo('name');
    if (name != null) {
      await processFiles(
          destinationPath, 'flutter_clean_architecture', name, false);
    }

    final nameOfFeature = featureName.toLowerCase();
    String pascalCase = toPascalCase(nameOfFeature);
    final filePath = 'lib/core/di/';
    final repoImport =
        "import 'package:${name ?? ''}/features/$nameOfFeature/domain/repositories/${nameOfFeature}_repo.dart';";

    final diGetterImports = [repoImport];
    final diGetterStatements = ["${pascalCase}Repo get${pascalCase}Repo();"];
    await addImportAndChangesInFile(
        '$filePath/di_getter.dart', diGetterImports, diGetterStatements);

    final mockDiGetterImports = [
      "import 'package:${name ?? ''}/features/$nameOfFeature/data/repository/mock_${nameOfFeature}_repo_impl.dart';",
      "import 'package:$name/core/di/injection_container.dart';",
      repoImport
    ];
    final mockDiGetterStatements = [
      "${pascalCase}Repo get${pascalCase}Repo() => Mock${pascalCase}RepoImpl(sl(), sl());",
    ];
    await addImportAndChangesInFile('$filePath/mock_di_getter.dart',
        mockDiGetterImports, mockDiGetterStatements);

    final prodDiGetterImports = [
      "import 'package:${name ?? ''}/features/$nameOfFeature/data/repository/${nameOfFeature}_repo_impl.dart';",
      repoImport
    ];
    final prodDiGetterStatements = [
      "${pascalCase}Repo get${pascalCase}Repo() => ${pascalCase}RepoImpl(sl(), sl());"
    ];
    await addImportAndChangesInFile('$filePath/prod_di_getter.dart',
        prodDiGetterImports, prodDiGetterStatements);

    final injectionContainerImports = [
      "import 'package:${name ?? ''}/features/$nameOfFeature/data/data_source/${nameOfFeature}_data_source.dart';",
      "import 'package:${name ?? ''}/features/$nameOfFeature/data/data_source/${nameOfFeature}_data_source_impl.dart';"
    ];
    final injectionContainerStatements = [
      "sl.registerLazySingleton<${pascalCase}DataSource>("
          "() => ${pascalCase}DataSourceImpl(remoteDataSource: sl()),"
          ");"
    ];
    await addImportAndChangesInFile('$filePath/injection_container.dart',
        injectionContainerImports, injectionContainerStatements);
    print('Feature $featureName created');
  } catch (e) {
    print('Error copying folder: $e');
  }
}

Future<void> addImportAndChangesInFile(String filePath,
    List<String> importStatements, List<String> methodCalls) async {
  final file = File(filePath);
  final lines = await file.readAsLines();

  int lastImportIndex = -1;
  for (int i = 0; i < lines.length; i++) {
    if (lines[i].startsWith("import ")) {
      lastImportIndex = i;
    } else {
      break;
    }
  }

  if (lastImportIndex != -1) {
    if (importStatements.isNotEmpty) {
      lines.insertAll(lastImportIndex + 1, importStatements);
    }
    int completionIndex = -1;
    for (int i = 0; i < lines.length; i++) {
      if (lines[i].trim() == '}') {
        completionIndex = i;
      }
    }

    if (completionIndex != -1) {
      if (methodCalls.isNotEmpty) {
        lines.insertAll(completionIndex, methodCalls);
      }
      await file.writeAsString(lines.join('\n'));
    }
  }
}
